function [Iout, vResidual, vPsnr] = admm_nlm_fun(Iref, frames, frameLocations, lambda, rho, numIter)
% ADMM-NLM function for superresolution problem
%
% @inputs
% Iref: original image to calculate PSNR
% frames: LR frames
% frameLocations: array indicating LR frame locations
% lambda: hyperparameter
% rho: hyperparameter
% numIter: maximum iteration for ADMM
%
% @outputs
% Iout: reconstructed image
% vResidual: vector containing residual for each iteration
% vPsnr: vector containing PSNR for each iteration

    resolution = size(frames(:,:,1)) * 3 + 2;   % output resolution
    kappa = lambda / rho;
    % Parameter initialization
    x = zeros(resolution);
    z = zeros(size(x));
    u = zeros(size(z));
    % A~ for x update
    AtildeFun = @(x) reshape(Atfun(Afun(reshape(x,resolution),frameLocations)...
        ,frameLocations) + rho * reshape(x,resolution),[],1);
    % CG hyperparameters
    maxIterCG = 20;
    tolCG = 1e-12;
    options.filterstrength=sqrt(kappa);
    % report results
    vPsnr = zeros(1,numIter);
    vResidual = zeros(1,numIter);
    % Main loop
    for k=1:numIter
        % x update
        v = z - u;
        btilde = Atfun(frames, frameLocations) + rho * v;
        x_vector = pcg(AtildeFun,btilde(:),tolCG,maxIterCG,[],[],x(:));
        x = reshape(x_vector, resolution);
        % z update
        v = x + u;
        z = NLMF(v,options);
        % u update
        u = u + x - z;
        % residual
        difference = Afun(x,frameLocations) - frames;
        res = 0.5 * sum(difference(:).^2,'all')
        vResidual(k) = res;
        % psnr
        vPsnr(k) = psnr(x,Iref);
    end
    Iout = x;
end