function [Iout, vResidual, vPsnr] = admm_tv_fun(Iref, frames, frameLocations, lambda, rho, numIter, isotropic)
% ADMM-TV function for superresolution problem
%
% @inputs
% Iref: original image to calculate PSNR
% frames: LR frames
% frameLocations: array indicating LR frame locations
% lambda: hyperparameter
% rho: hyperparameter
% numIter: maximum iteration for ADMM
% isotropic: bool, indicating isotropic or anisotropic TV
%
% @outputs
% Iout: reconstructed image
% vResidual: vector containing residual for each iteration
% vPsnr: vector containing PSNR for each iteration

    resolution = size(frames(:,:,1)) * 3 + 2;   % output resolution
    kappa = lambda / rho;
    % Parameter initialization
    z = zeros([resolution 2]);
    u = zeros(size(z));
    x = zeros(resolution);
    % A~ for x update
    AtildeFun = @(x) reshape(Atfun(Afun(reshape(x,resolution),frameLocations)...
        ,frameLocations) + rho * opDtx(opDx(reshape(x,resolution))),[],1);
    % CG hyperparameters
    maxIterCG = 20;
    tolCG = 1e-12;
    % Report results
    vResidual = zeros(1,numIter);
    vPsnr = zeros(1,numIter);
    % Main loop
    for k=1:numIter
        % x update
        v = z - u;
        btilde = Atfun(frames, frameLocations) + rho * opDtx(v);
        x_vector = pcg(AtildeFun,btilde(:),tolCG,maxIterCG,[],[],x(:));
        x = reshape(x_vector, resolution);
        % z update
            v = opDx(x) + u;
        if isotropic
            z = max(1-kappa./sqrt(v(:,:,1).^2+v(:,:,2).^2),0).*v;
        else
            z = max(v-kappa,0) + min(v+kappa,0);
        end
        % u update
        u = u + opDx(x) - z;
        % residual
        difference = Afun(x,frameLocations) - frames;
        res = 0.5 * sum(difference(:).^2) + lambda * sum(abs(opDx(x)),'all')
        vResidual(k) = res;
        % psnr
        vPsnr(k) = psnr(x,Iref);
    end
    Iout = x;
end
